@@ -6,8 +6,7 @@ import sys
 sys.path.insert(0,"/usr/lib/dialcentral/")
 
 
-import gc_dialer
+import dc_glade
 
 
-gc_dialer.run_dialpad()
-
+dc_glade.run_dialpad()
