@@ -39,7 +39,6 @@ import ConfigParser
 import itertools
 import warnings
 
-import gobject
 import gtk
 import gtk.glade
 
@@ -71,9 +70,9 @@ def display_error_message(msg):
 class Dialcentral(object):
 
 	_glade_files = [
-		'/usr/lib/dialcentral/dialcentral.glade',
 		os.path.join(os.path.dirname(__file__), "dialcentral.glade"),
 		os.path.join(os.path.dirname(__file__), "../lib/dialcentral.glade"),
+		'/usr/lib/dialcentral/dialcentral.glade',
 	]
 
 	KEYPAD_TAB = 0
@@ -87,16 +86,12 @@ class Dialcentral(object):
 	GV_BACKEND = 2
 	BACKENDS = (NULL_BACKEND, GC_BACKEND, GV_BACKEND)
 
-	_data_path = os.path.join(os.path.expanduser("~"), ".dialcentral")
-	_user_settings = "%s/settings.ini" % _data_path
-
 	def __init__(self):
 		self._initDone = False
 		self._connection = None
 		self._osso = None
 		self._clipboard = gtk.clipboard_get()
 
-		self._deviceIsOnline = True
 		self._credentials = ("", "")
 		self._selectedBackendId = self.NULL_BACKEND
 		self._defaultBackendId = self.GC_BACKEND
@@ -106,7 +101,9 @@ class Dialcentral(object):
 		self._messagesViews = None
 		self._recentViews = None
 		self._contactsViews = None
-		self._tabHoldTimeoutId = None
+		self._alarmHandler = None
+		self._ledHandler = None
+		self._originalCurrentLabels = []
 
 		for path in self._glade_files:
 			if os.path.isfile(path):
@@ -153,7 +150,9 @@ class Dialcentral(object):
 		else:
 			pass # warnings.warn("No Hildon", UserWarning, 2)
 
-		self._window.set_title("%s" % constants.__pretty_app_name__)
+		# If under hildon, rely on the application name being shown
+		if hildon is None:
+			self._window.set_title("%s" % constants.__pretty_app_name__)
 
 		callbackMapping = {
 			"on_dialpad_quit": self._on_close,
@@ -211,6 +210,15 @@ class Dialcentral(object):
 			else:
 				pass # warnings.warn("No OSSO", UserWarning, 2)
 
+			try:
+				import alarm_handler
+				self._alarmHandler = alarm_handler.AlarmHandler()
+			except ImportError:
+				alarm_handler = None
+			if hildon is not None:
+				import led_handler
+				self._ledHandler = led_handler.LedHandler()
+
 			# Setup maemo specifics
 			try:
 				import conic
@@ -232,12 +240,12 @@ class Dialcentral(object):
 			import gc_views
 
 			try:
-				os.makedirs(self._data_path)
+				os.makedirs(constants._data_path_)
 			except OSError, e:
 				if e.errno != 17:
 					raise
-			gcCookiePath = os.path.join(self._data_path, "gc_cookies.txt")
-			gvCookiePath = os.path.join(self._data_path, "gv_cookies.txt")
+			gcCookiePath = os.path.join(constants._data_path_, "gc_cookies.txt")
+			gvCookiePath = os.path.join(constants._data_path_, "gv_cookies.txt")
 			self._defaultBackendId = self._guess_preferred_backend((
 				(self.GC_BACKEND, gcCookiePath),
 				(self.GV_BACKEND, gvCookiePath),
@@ -256,12 +264,14 @@ class Dialcentral(object):
 				})
 				self._accountViews.update({
 					self.GC_BACKEND: gc_views.AccountInfo(
-						self._widgetTree, self._phoneBackends[self.GC_BACKEND], self._errorDisplay
+						self._widgetTree, self._phoneBackends[self.GC_BACKEND], None, self._errorDisplay
 					),
 					self.GV_BACKEND: gc_views.AccountInfo(
-						self._widgetTree, self._phoneBackends[self.GV_BACKEND], self._errorDisplay
+						self._widgetTree, self._phoneBackends[self.GV_BACKEND], self._alarmHandler, self._errorDisplay
 					),
 				})
+				self._accountViews[self.GC_BACKEND].save_everything = lambda *args: None
+				self._accountViews[self.GV_BACKEND].save_everything = self._save_settings
 				self._recentViews.update({
 					self.GC_BACKEND: gc_views.RecentCallsView(
 						self._widgetTree, self._phoneBackends[self.GC_BACKEND], self._errorDisplay
@@ -286,7 +296,7 @@ class Dialcentral(object):
 				})
 
 			evoBackend = evo_backend.EvolutionAddressBook()
-			fsContactsPath = os.path.join(self._data_path, "contacts")
+			fsContactsPath = os.path.join(constants._data_path_, "contacts")
 			fileBackend = file_backend.FilesystemAddressBookFactory(fsContactsPath)
 			for backendId in (self.GV_BACKEND, self.GC_BACKEND):
 				self._dialpads[backendId].number_selected = self._select_action
@@ -312,24 +322,30 @@ class Dialcentral(object):
 				"on_about_activate": self._on_about_activate,
 			}
 			self._widgetTree.signal_autoconnect(callbackMapping)
-			self._notebook.connect("button-press-event", self._on_tab_press)
-			self._notebook.connect("button-release-event", self._on_tab_release)
+
+			with gtk_toolbox.gtk_lock():
+				self._originalCurrentLabels = [
+					self._notebook.get_tab_label(self._notebook.get_nth_page(pageIndex)).get_text()
+					for pageIndex in xrange(self._notebook.get_n_pages())
+				]
+				self._notebookTapHandler = gtk_toolbox.TapOrHold(self._notebook)
+				self._notebookTapHandler.enable()
+			self._notebookTapHandler.on_tap = self._reset_tab_refresh
+			self._notebookTapHandler.on_hold = self._on_tab_refresh
+			self._notebookTapHandler.on_holding = self._set_tab_refresh
+			self._notebookTapHandler.on_cancel = self._reset_tab_refresh
 
 			self._initDone = True
 
 			config = ConfigParser.SafeConfigParser()
-			config.read(self._user_settings)
+			config.read(constants._user_settings_)
 			with gtk_toolbox.gtk_lock():
 				self.load_settings(config)
 
 			self._spawn_attempt_login(2)
-		except StandardError, e:
-			warnings.warn(e.message, UserWarning, 2)
-		except BaseException, e:
-			try:
-				warnings.warn(e.message, UserWarning, 2)
-			finally:
-				raise
+		except Exception, e:
+			with gtk_toolbox.gtk_lock():
+				self._errorDisplay.push_exception()
 
 	def attempt_login(self, numOfAttempts = 10, force = False):
 		"""
@@ -340,8 +356,6 @@ class Dialcentral(object):
 		try:
 			assert 0 <= numOfAttempts, "That was pointless having 0 or less login attempts"
 			assert self._initDone, "Attempting login before app is fully loaded"
-			if not self._deviceIsOnline:
-				raise RuntimeError("Unable to login, device is not online")
 
 			serviceId = self.NULL_BACKEND
 			loggedIn = False
@@ -360,7 +374,7 @@ class Dialcentral(object):
 				self._change_loggedin_status(serviceId)
 		except StandardError, e:
 			with gtk_toolbox.gtk_lock():
-				self._errorDisplay.push_exception(e)
+				self._errorDisplay.push_exception()
 
 	def _spawn_attempt_login(self, *args):
 		self._loginSink.send(args)
@@ -370,8 +384,6 @@ class Dialcentral(object):
 		@note Thread agnostic
 		"""
 		assert self._initDone, "Attempting login before app is fully loaded"
-		if not self._deviceIsOnline:
-			raise RuntimeError("Unable to login, device is not online")
 
 		loggedIn = False
 		if not loggedIn:
@@ -490,10 +502,21 @@ class Dialcentral(object):
 				for blob in blobs
 			)
 			self._credentials = tuple(creds)
+
+			if self._alarmHandler is not None:
+				self._alarmHandler.load_settings(config, "alarm")
+		except ConfigParser.NoOptionError, e:
+			warnings.warn(
+				"Settings file %s is missing section %s" % (
+					constants._user_settings_,
+					e.section,
+				),
+				stacklevel=2
+			)
 		except ConfigParser.NoSectionError, e:
 			warnings.warn(
 				"Settings file %s is missing section %s" % (
-					self._user_settings,
+					constants._user_settings_,
 					e.section,
 				),
 				stacklevel=2
@@ -509,10 +532,18 @@ class Dialcentral(object):
 			sectionName = "%s - %s" % (backendId, view.name())
 			try:
 				view.load_settings(config, sectionName)
+			except ConfigParser.NoOptionError, e:
+				warnings.warn(
+					"Settings file %s is missing section %s" % (
+						constants._user_settings_,
+						e.section,
+					),
+					stacklevel=2
+				)
 			except ConfigParser.NoSectionError, e:
 				warnings.warn(
 					"Settings file %s is missing section %s" % (
-						self._user_settings,
+						constants._user_settings_,
 						e.section,
 					),
 					stacklevel=2
@@ -527,6 +558,10 @@ class Dialcentral(object):
 		for i, value in enumerate(self._credentials):
 			blob = base64.b64encode(value)
 			config.set(constants.__pretty_app_name__, "bin_blob_%i" % i, blob)
+		config.add_section("alarm")
+		if self._alarmHandler is not None:
+			self._alarmHandler.save_settings(config, "alarm")
+
 		for backendId, view in itertools.chain(
 			self._dialpads.iteritems(),
 			self._accountViews.iteritems(),
@@ -552,18 +587,22 @@ class Dialcentral(object):
 		"""
 		config = ConfigParser.SafeConfigParser()
 		self.save_settings(config)
-		with open(self._user_settings, "wb") as configFile:
+		with open(constants._user_settings_, "wb") as configFile:
 			config.write(configFile)
 
 	def _refresh_active_tab(self):
-		page_num = self._notebook.get_current_page()
-		if page_num == self.CONTACTS_TAB:
+		pageIndex = self._notebook.get_current_page()
+		if pageIndex == self.CONTACTS_TAB:
 			self._contactsViews[self._selectedBackendId].update(force=True)
-		elif page_num == self.RECENT_TAB:
+		elif pageIndex == self.RECENT_TAB:
 			self._recentViews[self._selectedBackendId].update(force=True)
-		elif page_num == self.MESSAGES_TAB:
+		elif pageIndex == self.MESSAGES_TAB:
 			self._messagesViews[self._selectedBackendId].update(force=True)
 
+		if pageIndex in (self.RECENT_TAB, self.MESSAGES_TAB):
+			if self._ledHandler is not None:
+				self._ledHandler.off()
+
 	def _on_close(self, *args, **kwds):
 		try:
 			if self._osso is not None:
@@ -602,11 +641,9 @@ class Dialcentral(object):
 		bearer = event.get_bearer_type()
 
 		if status == conic.STATUS_CONNECTED:
-			self._deviceIsOnline = True
 			if self._initDone:
 				self._spawn_attempt_login(2)
 		elif status == conic.STATUS_DISCONNECTED:
-			self._deviceIsOnline = False
 			if self._initDone:
 				self._defaultBackendId = self._selectedBackendId
 				self._change_loggedin_status(self.NULL_BACKEND)
@@ -640,37 +677,46 @@ class Dialcentral(object):
 
 		self._spawn_attempt_login(2, True)
 
-	def _on_notebook_switch_page(self, notebook, page, page_num):
-		if page_num == self.RECENT_TAB:
-			self._recentViews[self._selectedBackendId].update()
-		elif page_num == self.MESSAGES_TAB:
-			self._messagesViews[self._selectedBackendId].update()
-		elif page_num == self.CONTACTS_TAB:
+	def _on_notebook_switch_page(self, notebook, page, pageIndex):
+		self._reset_tab_refresh()
+		if pageIndex == self.RECENT_TAB:
+			didRecentUpdate = self._recentViews[self._selectedBackendId].update()
+		elif pageIndex == self.MESSAGES_TAB:
+			didMessagesUpdate = self._messagesViews[self._selectedBackendId].update()
+		elif pageIndex == self.CONTACTS_TAB:
 			self._contactsViews[self._selectedBackendId].update()
-		elif page_num == self.ACCOUNT_TAB:
+		elif pageIndex == self.ACCOUNT_TAB:
 			self._accountViews[self._selectedBackendId].update()
 
-	def _on_tab_press(self, *args):
-		self._tabHoldTimeoutId = gobject.timeout_add(1000, self._on_tab_refresh)
+		if didRecentUpdate or didMessagesUpdate:
+			if self._ledHandler is not None:
+				self._ledHandler.off()
 
-	def _on_tab_release(self, *args):
-		if self._tabHoldTimeoutId is not None:
-			gobject.source_remove(self._tabHoldTimeoutId)
-		self._tabHoldTimeoutId = None
+	def _set_tab_refresh(self, *args):
+		pageIndex = self._notebook.get_current_page()
+		child = self._notebook.get_nth_page(pageIndex)
+		self._notebook.get_tab_label(child).set_text("Refresh?")
+		return False
+
+	def _reset_tab_refresh(self, *args):
+		pageIndex = self._notebook.get_current_page()
+		child = self._notebook.get_nth_page(pageIndex)
+		self._notebook.get_tab_label(child).set_text(self._originalCurrentLabels[pageIndex])
+		return False
 
 	def _on_tab_refresh(self, *args):
-		self._tabHoldTimeoutId = None
 		self._refresh_active_tab()
+		self._reset_tab_refresh()
 		return False
 
 	def _on_sms_clicked(self, number, message):
-		assert number
-		assert message
+		assert number, "No number specified"
+		assert message, "Empty message"
 		try:
 			loggedIn = self._phoneBackends[self._selectedBackendId].is_authed()
 		except StandardError, e:
 			loggedIn = False
-			self._errorDisplay.push_exception(e)
+			self._errorDisplay.push_exception()
 			return
 
 		if not loggedIn:
@@ -684,9 +730,9 @@ class Dialcentral(object):
 			self._phoneBackends[self._selectedBackendId].send_sms(number, message)
 			dialed = True
 		except StandardError, e:
-			self._errorDisplay.push_exception(e)
+			self._errorDisplay.push_exception()
 		except ValueError, e:
-			self._errorDisplay.push_exception(e)
+			self._errorDisplay.push_exception()
 
 	def _on_dial_clicked(self, number):
 		assert number, "No number to call"
@@ -694,7 +740,7 @@ class Dialcentral(object):
 			loggedIn = self._phoneBackends[self._selectedBackendId].is_authed()
 		except StandardError, e:
 			loggedIn = False
-			self._errorDisplay.push_exception(e)
+			self._errorDisplay.push_exception()
 			return
 
 		if not loggedIn:
@@ -709,9 +755,9 @@ class Dialcentral(object):
 			self._phoneBackends[self._selectedBackendId].dial(number)
 			dialed = True
 		except StandardError, e:
-			self._errorDisplay.push_exception(e)
+			self._errorDisplay.push_exception()
 		except ValueError, e:
-			self._errorDisplay.push_exception(e)
+			self._errorDisplay.push_exception()
 
 		if dialed:
 			self._dialpads[self._selectedBackendId].clear()
@@ -747,11 +793,14 @@ def run_doctest():
 
 
 def run_dialpad():
-	gtk.gdk.threads_init()
-	if hildon is not None:
-		gtk.set_application_name(constants.__pretty_app_name__)
-	handle = Dialcentral()
-	gtk.main()
+	_lock_file = os.path.join(constants._data_path_, ".lock")
+	with gtk_toolbox.flock(_lock_file, 0):
+		gtk.gdk.threads_init()
+
+		if hildon is not None:
+			gtk.set_application_name(constants.__pretty_app_name__)
+		handle = Dialcentral()
+		gtk.main()
 
 
 class DummyOptions(object):
